"""Reward hack result types are defined in afterburn.types.

This module re-exports them for convenience.
"""

from afterburn.types import (
    FormatGamingResult,
    LengthBiasResult,
    RewardHackResult,
    RiskLevel,
    StrategyCollapseResult,
    SycophancyResult,
)

__all__ = [
    "FormatGamingResult",
    "LengthBiasResult",
    "RewardHackResult",
    "RiskLevel",
    "StrategyCollapseResult",
    "SycophancyResult",
]
