"""Behaviour result types are defined in afterburn.types.

This module re-exports them for convenience.
"""

from afterburn.types import (
    BehaviourResult,
    CalibrationAnalysis,
    ChainOfThoughtAnalysis,
    FormatAnalysis,
    LengthAnalysis,
    StrategyShiftAnalysis,
)

__all__ = [
    "BehaviourResult",
    "CalibrationAnalysis",
    "ChainOfThoughtAnalysis",
    "FormatAnalysis",
    "LengthAnalysis",
    "StrategyShiftAnalysis",
]
